
local EPGPLoot = {}
EPGPLoot.__index = EPGPLoot

EPGPLoot.MODE_EPGP = "epgp"
EPGPLoot.MODE_ROLL = "roll"

function EPGPLoot.new()
    local self = {}
    setmetatable(self, EPGPLoot)
    self.currentMode = self.MODE_ROLL
    self.bufferedData = {}
    self.LH = C_LootHistory
    return self
end

function EPGPLoot:isEPGPMode()
    return (self.currentMode == self.MODE_EPGP)
end

function EPGPLoot:LoadPlayerInfo(itemIdx, playerIdx)

    local rollID = self.LH.GetItem(itemIdx);
    local name, _, rollType = self.LH.GetPlayerInfo(itemIdx, playerIdx);
    local currentData

    if (not self.bufferedData[rollID]) then
        self.bufferedData[rollID] = {}
    end

    if (not self.bufferedData[rollID][name]) then
        local ep,gp = EPGP:GetEPGP(name)
        currentData = {}
        currentData.EPGP = 0
        currentData.hasNotMin = true
        if (ep and gp and gp > 0) then
            currentData.EPGP = (ep/gp)
        end
        if (ep) then
            currentData.hasNotMin = (EPGP:GetMinEP() > ep)
        end
        self.bufferedData[rollID][name] = currentData
    else
        currentData = self.bufferedData[rollID][name]
    end

    return currentData.EPGP, currentData.hasNotMin, rollType
end

function EPGPLoot:compareFramesByEPGP (f1, f2)
    if (f1 and f2 and f1.playerIdx and f2.playerIdx ) then

        local firstEPGP, firstHasNotMin, firstRollType = self:LoadPlayerInfo(f1.itemIdx, f1.playerIdx)
        local secondEPGP, secondHasNotMin, secondRollType = self:LoadPlayerInfo(f2.itemIdx, f2.playerIdx)

        if (firstRollType == secondRollType) then
            if ((firstHasNotMin == false and secondHasNotMin == true) or (firstEPGP > secondEPGP)) then
                return true
            end
        elseif (firstRollType == LOOT_ROLL_TYPE_NEED) then
            return true
        elseif (firstRollType == LOOT_ROLL_TYPE_GREED and (secondRollType == LOOT_ROLL_TYPE_DISENCHANT or secondRollType == LOOT_ROLL_TYPE_PASS)) then
            return true
        elseif (firstRollType == LOOT_ROLL_TYPE_DISENCHANT and secondRollType == LOOT_ROLL_TYPE_PASS) then
            return true
        end
    elseif (f1 and f1.playerIdx) then
        return true
    elseif (not f1.playerIdx and not f2.playerIdx) then
        return true
    end
end

local origUpdatePlayerFrame = LootHistoryFrame_UpdatePlayerFrame
function LootHistoryFrame_UpdatePlayerFrame(self, playerFrame)

    local origResult = origUpdatePlayerFrame(self,playerFrame)
    if (playerFrame.playerIdx and self.LootModeHandle) then
        local lootHandle = self.LootModeHandle

        if (lootHandle:isEPGPMode()) then
            -- load player data
            local epgp, hasNotMinEp, rollType = lootHandle:LoadPlayerInfo(playerFrame.itemIdx, playerFrame.playerIdx)
            if (rollType and (rollType == LOOT_ROLL_TYPE_NEED or rollType == LOOT_ROLL_TYPE_GREED or rollType == LOOT_ROLL_TYPE_DISENCHANT)) then
                local text = string.format(" %.2f", epgp)
                if (hasNotMinEp == true) then
                    text = '|cff555555'..text..'|r'
                end

                playerFrame.RollText:SetText(text)
                playerFrame.RollText:SetSize(0,16)
                playerFrame.RollIcon:SetPoint("RIGHT", playerFrame.RollText, "LEFT", 2, -1);
            end
        else
            -- reset to default options
            playerFrame.RollText:SetSize(24,16)
        end
    end

    return origResult
end

local origUpdatePlayerFrames = LootHistoryFrame_UpdatePlayerFrames
function LootHistoryFrame_UpdatePlayerFrames(self, itemIdx)

    local firstFrame, lastFrame = origUpdatePlayerFrames(self, itemIdx)

    if (self.LootModeHandle and self.LootModeHandle:isEPGPMode()) then
       local frameList = {}

       -- build lookup table of player frames
       local currentFrame = lastFrame
       repeat
           if (currentFrame.playerIdx) then
               table.insert(frameList, currentFrame)
           end
           local pointCount = currentFrame:GetNumPoints()
           if (pointCount > 0) then
               local point, relativeTo = currentFrame:GetPoint(n)
               currentFrame = relativeTo
           end
       until (pointCount == 0)


       table.sort(frameList, function(f1, f2) return LootHistoryFrame.LootModeHandle:compareFramesByEPGP(f1, f2) end )

       firstFrame = nil
       lastFrame = nil
       for i=1,#frameList do
           local playerFrame = frameList[i]
           playerFrame:ClearAllPoints()
           if ( lastFrame ) then
               playerFrame:SetPoint("TOPLEFT", lastFrame, "BOTTOMLEFT", 0, -1);
           end
           if ( i % 2 == 1) then
               playerFrame.AlternatingBG:Show();
           else
               playerFrame.AlternatingBG:Hide();
           end

           lastFrame = playerFrame
           firstFrame = firstFrame or playerFrame;
       end
    end

    return firstFrame, lastFrame
end

-- region update priorities after loot distribution

-- refresh PRs of undistributed items
-- TODO: C_LootHistory.CanMasterLoot doesn't work for non master looter
-- TODO: at first focus on master looter
function EPGPLoot:refreshRemainingItems()
    local numItems = self.LH.GetNumItems()
    for itemIdx=1, numItems do
        local rollID, itemLink, numPlayers = self.LH.GetItem(itemIdx)
        if (numPlayers > 0 and LootHistoryFrame.expandedRolls[rollID] and self.LH.CanMasterLoot(itemIdx, 1)) then
            self.bufferedData[rollID] = {}
            LootHistoryFrame_UpdatePlayerFrames(LootHistoryFrame, itemIdx)
        end
    end
end
-- endregion

function EPGPLoot:UpdateBtnTexture(button, mode)

    if (mode == self.MODE_EPGP) then
        button:SetNormalTexture("Interface\\GossipFrame\\VendorGossipIcon")
    else
        button:SetNormalTexture("Interface\\Buttons\\UI-GroupLoot-Dice-Up")
    end
end

function EPGPLoot:ToggleMode(button)

    if (self.currentMode == self.MODE_ROLL) then
        self.currentMode = self.MODE_EPGP
    else
        self.currentMode = self.MODE_ROLL
    end

    self:UpdateBtnTexture(button, self.currentMode)
    LootHistoryFrame_FullUpdate(LootHistoryFrame)
end

local function BuildToggleBtn(frame, handler)

    local btn = CreateFrame("Button","ToggleLootModeBtn",frame)
    btn:SetPoint("TOPRIGHT",frame,"TOPLEFT",2,-8)
    btn:SetSize(16, 16)
    handler:UpdateBtnTexture(btn)
    btn:SetScript('OnClick', function(btn) LootHistoryFrame.LootModeHandle:ToggleMode(btn) end)

    return btn
end

LootHistoryFrame:HookScript("OnShow", function(self)
    if (not self.ToggleLootModeBtn) then
        local lootHandler = EPGPLoot.new()
        self.LootModeHandle = lootHandler
        self.ToggleLootModeBtn = BuildToggleBtn(self.CloseButton, lootHandler)

        local LLNLib = LibStub:GetLibrary("LibLootNotify-1.0")
        LLNLib.RegisterCallback(EPGP, "LootReceived", function() LootHistoryFrame.LootModeHandle:refreshRemainingItems() end)
    end
end)

-- overwrite original function
-- hide passes as master looter to clean the list a bit
function LootHistoryFrameUtil_ShouldDisplayPlayer(itemIdx, playerIdx)
    local rollID, itemLink, numPlayers, isDone, winnerIdx, isMasterLoot = C_LootHistory.GetItem(itemIdx);
    local name, class, rollType, roll, isWinner, isMe = C_LootHistory.GetPlayerInfo(itemIdx, playerIdx);

    return isMe or roll or (not isDone) or (isMasterLoot and rollType ~= LOOT_ROLL_TYPE_PASS);
end
